#!/bin/bash

CLICKSIGNPATH=$HOME/clicksign
CLICKSIGNTMP="/tmp/clicksigntmpdir"

clicksigninstall() #Installer function
{
    echo "Installing clicksign"
    copycs
    if [ $? -eq 0 ]
    then
        mkdeskall
        mvdesk
        sudo cp $HOME/clicksign/icons/clicksign.png /usr/share/icons/
        mkdirentry
        addmenu
        mkmimeinfo
        if [ ! -d $CLICKSIGNPATH/licenses ]
        then
            mkdir $CLICKSIGNPATH/licenses
        fi
        echo "DONE, clicksign has been installed on $HOME"
    else
        echo "Clicksign could not be installed"
        echo "ERROR: $?"
    fi
    chmod -R +x $CLICKSIGNPATH
    chmod -R +x $HOME/.local/share/applications/clicksign*
}

clicksignremove() #Uninstaller function
{
    echo "Really do you want to uninstall clicksign?[y/n]:"
    read opt
    case $opt in
    [yY])
        rmclicksignhome
        rm $HOME/.local/share/applications/clicksign* 2> /dev/null
        echo "DONE"
        ;;
    [nN])
        echo "Operation aborted"
        ;;
    *)
        echo "Incorrect option, operation aborted"
        ;;
    esac
}

clicksignremoveall() #Uninstaller function
{
    echo "Really do you want to uninstall clicksign and configuration files?[y/n]:"
    read opt
    case $opt in
    [yY])
        rm -r $CLICKSIGNPATH
        rm $HOME/.local/share/applications/clicksign* 2> /dev/null
        echo "DONE"
        ;;
    [nN])
        echo "Operation aborted"
        ;;
    *)
        echo "Incorrect option, operation aborted"
        ;;
    esac
}

rmclicksignhome() #Removes the data that must be removed
{
    savedata
    rm -r $CLICKSIGNPATH
    mv $CLICKSIGNTMP $CLICKSIGNPATH
}

savedata() #Saves the data that will not be removed
{
    if [ -d $CLICKSIGNTMP ]
    then
        rm -r $CLICKSIGNTMP
    fi
    mkdir $CLICKSIGNTMP
    cp -r $CLICKSIGNPATH/ism.* $CLICKSIGNTMP/
    cp -r $CLICKSIGNPATH/inbox $CLICKSIGNTMP/
    cp -r $CLICKSIGNPATH/outbox $CLICKSIGNTMP/
    cp -r $CLICKSIGNPATH/ids $CLICKSIGNTMP/
    cp -r $CLICKSIGNPATH/licenses $CLICKSIGNTMP/
}

clicksignreinstall()
{
    savedata
    clicksigninstall
    mv $CLICKSIGNTMP/* $CLICKSIGNPATH/ 2> /dev/null
}

copycs(){ #Copy the clicksign directory to the user's HOME
    dest="$HOME/clicksign"
    if [ ! -d $dest ]
    then
        mkdir $dest
    fi
    for f in `ls | grep -v install`
    do
        cp -r $f $dest   
    done
}

mkdesk() #Makes the .desktop file's base
{
    # $1 == EXEC
    # $2 == ICON
    # $3 == NAME
    # $4 == TARGET
    echo "#!/usr/bin/env xdg-open" > $4
    echo "[Desktop Entry]" >> $4
    echo "Encoding=UTF-8" >> $4
    echo "Type=Application" >> $4
    echo "Exec=$1" >> $4
    echo "Icon=$2" >> $4
    echo "Name=$3" >> $4
    echo "Categories=Clicksign" >> $4
}

mkdeskabout() #.desktop for about
{
    mkdesk $CLICKSIGNPATH/menu/about.sh $CLICKSIGNPATH/icons/comments.png "About" $CLICKSIGNPATH/clicksign-about.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/clicksign-about.desktop
    echo "Name[es]=Acerca de" >> $CLICKSIGNPATH/clicksign-about.desktop
    echo "Name[ca]=Quant a..." >> $CLICKSIGNPATH/clicksign-about.desktop
}

mkdesksetup() #.desktop for setup
{
    mkdesk $CLICKSIGNPATH/menu/setup.sh $CLICKSIGNPATH/icons/application.png "Setup" $CLICKSIGNPATH/clicksign-setup.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/clicksign-setup.desktop
    echo "Name[es]=Configuración" >> $CLICKSIGNPATH/clicksign-setup.desktop
    echo "Name[ca]=Configuració" >> $CLICKSIGNPATH/clicksign-setup.desktop
}

mkdeskverify() #.desktop for verify
{
    mkdesk "$CLICKSIGNPATH/menu/verify.sh %F" $CLICKSIGNPATH/icons/verify.png "Clicksign Verify" $CLICKSIGNPATH/clicksign-verify.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/clicksign-verify.desktop
    echo "Name[es]=Clicksign Verificar" >> $CLICKSIGNPATH/clicksign-verify.desktop
    echo "Name[ca]=Clicksign Verificar" >> $CLICKSIGNPATH/clicksign-verify.desktop
    echo "NoDisplay=true" >> $CLICKSIGNPATH/clicksign-verify.desktop
}

mkdesksign() #.desktop for sign
{
    mkdesk "$CLICKSIGNPATH/menu/sign.sh %F" $CLICKSIGNPATH/icons/sign.png "Clicksign Sign" $CLICKSIGNPATH/clicksign-sign.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/clicksign-sign.desktop
    echo "Name[es]=Clicksign Firmar" >> $CLICKSIGNPATH/clicksign-sign.desktop
    echo "Name[ca]=Clicksign Signar" >> $CLICKSIGNPATH/clicksign-sign.desktop
    echo "NoDisplay=true" >> $CLICKSIGNPATH/clicksign-sign.desktop
}

mkdeskportasigma() #.desktop for portasigma uploader
{
    mkdesk "$CLICKSIGNPATH/menu/portasigma-upload.sh %F" $CLICKSIGNPATH/icons/portasigma.png "Upload to PortaSigma" $CLICKSIGNPATH/portasigma-upload.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/portasigma-upload.desktop
    echo "Name[es]=Subir a PortaSigma" >> $CLICKSIGNPATH/portasigma-upload.desktop
    echo "Name[ca]=Pujar a PortaSigma" >> $CLICKSIGNPATH/portasigma-upload.desktop
    echo "NoDisplay=true" >> $CLICKSIGNPATH/portasigma-upload.desktop
}

mkdesksignloop() #.desktop for signloop
{
    mkdesk $CLICKSIGNPATH/menu/signloop.sh $CLICKSIGNPATH/icons/signloop.png "Signloop" $CLICKSIGNPATH/clicksign-signloop.desktop
    echo "MimeType=application/pdf;application/x-pdfsignable;" >> $CLICKSIGNPATH/clicksign-signloop.desktop
}

mkdeskall() #Makes all .desktop
{
    mkdeskabout
    mkdesksetup
    mkdeskverify
    mkdesksign
    mkdeskportasigma
    mkdesksignloop
}

mvdesk() #Moves the specific .desktop to the menu
{
    mv $CLICKSIGNPATH/clicksign-about.desktop $HOME/.local/share/applications/clicksign-about.desktop
    mv $CLICKSIGNPATH/clicksign-setup.desktop $HOME/.local/share/applications/clicksign-setup.desktop
    mv $CLICKSIGNPATH/clicksign-signloop.desktop $HOME/.local/share/applications/clicksign-signloop.desktop
    mv $CLICKSIGNPATH/clicksign-sign.desktop $HOME/.local/share/applications/clicksign-sign.desktop
    mv $CLICKSIGNPATH/portasigma-upload.desktop $HOME/.local/share/applications/portasigma-upload.desktop
    mv $CLICKSIGNPATH/clicksign-verify.desktop $HOME/.local/share/applications/clicksign-verify.desktop
}


mkdirentry() #Makes the Clicksign .directory
{
    sudo sh -c 'echo "[Desktop Entry]" > /usr/share/desktop-directories/Clicksign.directory'
    sudo sh -c 'echo "Name=Clicksign" >> /usr/share/desktop-directories/Clicksign.directory'
    sudo sh -c 'echo "Comment=Clicksign menu" >> /usr/share/desktop-directories/Clicksign.directory'
    sudo sh -c 'echo "Icon=/usr/share/icons/clicksign.png" >> /usr/share/desktop-directories/Clicksign.directory'
    sudo sh -c 'echo "Type=Directory" >> /usr/share/desktop-directories/Clicksign.directory'
    sudo sh -c 'echo "X-Ubuntu-Gettext-Domain=gnome-menus" >> /usr/share/desktop-directories/Clicksign.directory'
}

mkmenu() #Makes the clicksign menu on the .menu temporal file
{
    sudo sh -c 'echo "<!-- Clicksign menu -->" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Menu>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Name>Clicksign</Name>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Directory>Clicksign.directory</Directory>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Include>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<And>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Category>Clicksign</Category>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Not>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "<Category>KDE</Category>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "</Not>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "</And>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "</Include>" >> /tmp/applicationstmp'
    sudo sh -c 'echo "</Menu> <!-- End Clicksign -->" >> /tmp/applicationstmp'
}

addgmenu() #Adds the Clicksign menu if neccessary on GNOME applications menu file
{
    grep Clicksign /etc/xdg/menus/applications.menu > /dev/null 2>/dev/null
    if [ ! $? -eq 0 ]
    then
        if [ -f /tmp/applicationstmp ]
        then
            sudo rm /tmp/applicationstmp
        fi
        grep -v "</Menu> <\!-- End Applications -->" /etc/xdg/menus/applications.menu > /tmp/applicationstmp
        mkmenu
        grep "</Menu> <\!-- End Applications -->" /etc/xdg/menus/applications.menu >> /tmp/applicationstmp
        sudo sh -c 'cat /tmp/applicationstmp > /etc/xdg/menus/applications.menu'
    fi
}

addkmenu() #Adds the Clicksign menu if necessary on the KDE applications menu file
{
    grep Clicksign /etc/xdg/menus/kde-applications.menu > /dev/null 2>/dev/null
    if [ ! $? -eq 0 ]
    then
        if [ -f /tmp/applicationstmp ]
        then
            sudo rm /tmp/applicationstmp
        fi
        sed '$ d' /etc/xdg/menus/kde-applications.menu > /tmp/applicationstmp
        mkmenu
        echo "</Menu>" >> /tmp/applicationstmp
        sudo sh -c 'cat /tmp/applicationstmp > /etc/xdg/menus/kde-applications.menu'
    fi
}

addk4menu() #Adds the Clicksign menu if necessary on the KDE4 applications menu file
{
    grep Clicksign /etc/xdg/menus/kde4-applications.menu > /dev/null 2>/dev/null
    if [ ! $? -eq 0 ]
    then
        if [ -f /tmp/applicationstmp ]
        then
            sudo rm /tmp/applicationstmp
        fi
        sed '$ d' /etc/xdg/menus/kde4-applications.menu > /tmp/applicationstmp
        mkmenu
        echo "</Menu>" >> /tmp/applicationstmp
        sudo sh -c 'cat /tmp/applicationstmp > /etc/xdg/menus/kde4-applications.menu'
    fi
}

addmenu() #Adds the Clicksign menu on GNOME and KDE if exists
{
    if [ -f /etc/xdg/menus/applications.menu ]
    then
        addgmenu
    fi
    
    if [ -f /etc/xdg/menus/kde-applications.menu ]
    then
        addkmenu 
    fi
    
    if [ -f /etc/xdg/menus/kde4-applications.menu ]
    then
        addk4menu
    fi
}

mkmimeinfo() #Makes a MIMEinfo.cache with the aplications
{
    if [ ! -f $HOME/.local/share/applications/mimeinfo.cache ]
    then
        echo "[MIME Cache]" > $HOME/.local/share/applications/mimeinfo.cache
        echo "application/pdf=clicksign-verify.desktop;clicksign-sign.desktop;portasigma-upload.desktop;clicksign-signloop.desktop;clicksign-setup.desktop;" >> $HOME/.local/share/applications/mimeinfo.cache
    else
        echo "application/pdf=clicksign-verify.desktop;clicksign-sign.desktop;portasigma-upload.desktop;" >> $HOME/.local/share/applications/mimeinfo.cache
    fi
}

if [ ! -d $CLICKSIGNPATH ]
then
    clicksigninstall
else
    echo "Clicksign alredy exists on your HOME, maybe you have uninstalled it yet except the configuration files, you can uninstall all if you want"
    echo "Select an option:"
    echo "1.Uninstall"
    echo "2.Reinstall"
    echo "3.Uninstall all"
    echo "4.Abort"
    echo "Option[4]: "
    read option
    case $option in
    1)
        clicksignremove
        ;;
    2)
        clicksignreinstall
        ;;
    3)
        clicksignremoveall
        ;;
    *)
        echo "Operation aborted"
        ;;
    esac
    
fi
